@@ -1,13 +1,24 @@
-
 package AWS::S3::FileIterator;
 
 use strict;
 use warnings 'all';
-use base 'Iterator::Paged';
 use Carp 'confess';
 use AWS::S3::Owner;
 use AWS::S3::File;
 
+sub new {
+  my ($class, %args) = @_;
+
+  my $s = bless {
+    data        => [ ],
+    page_number => 0,
+    idx         => 0,
+    %args,
+  }, $class;
+  $s->_init;
+  return $s;
+}
+
 sub _init {
     my ( $s ) = @_;
 
@@ -27,6 +38,7 @@ sub marker    { shift->{marker} }
 sub pattern   { shift->{pattern} }
 sub bucket    { shift->{bucket} }
 sub page_size { shift->{page_size} }
+sub prefix    { shift->{prefix} }
 
 sub has_prev {
     my $s = shift;
@@ -36,6 +48,30 @@ sub has_prev {
 
 sub has_next { shift->{has_next} }
 
+sub next {
+  my $s = shift;
+
+  if( exists( $s->{data}->[ $s->{idx} ] ) ) {
+    return $s->{data}->[ $s->{idx}++ ];
+  } else {
+    # End of the current resultset, see if we can get another page of records:
+    if( my $page = $s->next_page ) {
+      $s->{data} = $page;
+      $s->{idx} = 0;
+      return $s->{data}->[ $s->{idx}++ ];
+    } else {
+      # No more pages, no more data:
+      return;
+    }
+  }
+}
+
+sub reset {
+  my $s = shift;
+  $s->{idx} = 0;
+}
+
+
 sub page_number {
     my $s = shift;
     @_ ? $s->{page_number} = $_[0] - 1 : $s->{page_number};
@@ -171,7 +207,8 @@ AWS::S3::FileIterator - Easily access and iterate through your S3 files.
     page_size   => 100,
     page_number => 1,
     # Optional params:
-    pattern     => qr(\.txt$)
+    pattern     => qr(\.txt$),
+    prefix      => 'notes',
   );
   
   while( my @files = $iter->next_page )
@@ -203,7 +240,7 @@ Boolean - read-only
 
 =head2 page_number
 
-Integer -  read-write
+Integer - read-write
 
 =head2 marker
 
@@ -217,6 +254,12 @@ Regexp - read-only
 
 If supplied to the constructor, only files which match the pattern will be returned.
 
+=head2 prefix
+
+String - read-only
+
+If supplied to the constructor, only files which begin with the indicated prefix will be returned.
+
 =head1 PUBLIC METHODS
 
 =head2 next_page()
