@@ -2,15 +2,14 @@
 package AWS::S3::HTTPRequest;
 
 use Moose;
+use Moose::Util::TypeConstraints;
 use AWS::S3::Signer;
 
-use Carp 'confess';
-use HTTP::Date 'time2str';
-use MIME::Base64 qw(encode_base64);
-use URI::Escape qw(uri_escape_utf8);
-use Digest::HMAC_SHA1;
+use HTTP::Headers;
 use URI;
 
+with 'AWS::S3::Roles::Bucket';
+
 my $METADATA_PREFIX      = 'x-amz-meta-';
 my $AMAZON_HEADER_PREFIX = 'x-amz-';
 
@@ -34,6 +33,12 @@ has 'path' => (
     isa      => 'Str',
 );
 
+class_type( 'HTTP::Headers' );
+
+coerce 'HTTP::Headers'
+    => from 'HashRef'
+    => via { my $h = HTTP::Headers->new( %$_ ) };
+
 has 'headers' => (
     is       => 'ro',
     required => 1,
@@ -43,8 +48,6 @@ has 'headers' => (
     coerce   => 1,
 );
 
-coerce 'HTTP::Headers' => from 'HashRef' => via { my $h = HTTP::Headers->new( %$_ ) };
-
 has 'content' => (
     is       => 'ro',
     required => 1,
@@ -69,24 +72,18 @@ has 'contenttype' => (
 sub http_request {
     my $s        = shift;
     my $method   = $s->method;
-    my $path     = $s->path;
     my $headers  = $s->headers;
     my $content  = $s->content;
     my $metadata = $s->metadata;
 
-    my $protocol = $s->s3->secure ? 'https' : 'http';
-	my $endpoint = $s->s3->endpoint;
-    my $uri = "$protocol://$endpoint/$path";
-    if ( $path =~ m{^([^/?]+)(.*)} && _is_dns_bucket( $1 ) ) {
-        $uri = "$protocol://$1.$endpoint$2";
-    }    # end if()
+    my $uri = $s->bucket_uri( $s->path );
 
     my $signer = AWS::S3::Signer->new(
         s3      => $s->s3,
         method  => $method,
         uri     => $uri,
         content => $content ? \$content : undef,
-        headers => $headers,
+        headers => [ $headers->flatten ],
     );
 
     $headers->header( 'Authorization'  => $signer->auth_header );
@@ -97,17 +94,8 @@ sub http_request {
 
     my $request = HTTP::Request->new( $method, $uri, $headers, $content );
 
-    if ( $uri =~ m{location} && 1 || $method eq 'PUT' ) {
-
-        #  warn "StringToSign(" . $signer->string_to_sign . ")";
-        #  warn "canonicalized_amz_headers(" . $signer->canonicalized_amz_headers . ")";
-        #  warn "Request(" . $request->as_string . ")";
-    }    # end if()
     return $request;
 }    # end http_request()
 
-# XXX: Not needed by us...
-sub _is_dns_bucket { 1 }
-
 __PACKAGE__->meta->make_immutable;
 
